
/* Chrysalide - Outil d'analyse de fichiers binaires
 * flat.c - équivalent Python du fichier "format/flat.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "flat.h"


#include <pygobject.h>


#include <format/flat.h>


#include "executable.h"
#include "../helpers.h"
#include "../analysis/content.h"



/* Crée un nouvel objet Python de type 'FlatFormat'. */
static PyObject *py_flat_format_new(PyTypeObject *, PyObject *, PyObject *);

/* Indique le type d'architecture visée par le format. */
static PyObject *py_flat_format_set_target_machine(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de l'objet à instancier.                         *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Crée un nouvel objet Python de type 'FlatFormat'.            *
*                                                                             *
*  Retour      : Instance Python mise en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_flat_format_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyObject *result;                       /* Instance à retourner        */
    PyObject *content_obj;                  /* Objet pour le contenu       */
    int ret;                                /* Bilan de lecture des args.  */
    GBinContent *content;                   /* Instance GLib du contenu    */
    GExeFormat *format;                     /* Création GLib à transmettre */

    ret = PyArg_ParseTuple(args, "O!", get_python_binary_content_type(), &content_obj);
    if (!ret) return NULL;

    content = G_BIN_CONTENT(pygobject_get(content_obj));

    format = g_flat_format_new(content);

    if (format == NULL)
    {
        result = Py_None;
        Py_INCREF(result);
    }

    else
    {
        result = pygobject_new(G_OBJECT(format));
        g_object_unref(format);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = description de l'exécutable à consulter.              *
*                args = arguments accompagnant l'appel.                       *
*                                                                             *
*  Description : Indique le type d'architecture visée par le format.          *
*                                                                             *
*  Retour      : Rien, donc None en Python.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_flat_format_set_target_machine(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    GFlatFormat *format;                    /* Version GLib du format      */
    const char *machine;                    /* Identifiant d'architecture  */
    int ret;                                /* Bilan de lecture des args.  */

    format = G_FLAT_FORMAT(pygobject_get(self));
    assert(format != NULL);

    ret = PyArg_ParseTuple(args, "s", &machine);
    if (!ret) return NULL;

    g_flat_format_set_target_machine(format, machine);

    result = Py_None;
    Py_INCREF(result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_flat_format_type(void)
{
    static PyMethodDef py_flat_format_methods[] = {
        {
            "set_machine", py_flat_format_set_target_machine,
            METH_VARARGS,
            "set_machine($self, machine, /)\n--\n\nSet the architecture linked to the flat format."
        },
        { NULL }
    };

    static PyGetSetDef py_flat_format_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_flat_format_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.format.FlatFormat",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide flat format",

        .tp_methods     = py_flat_format_methods,
        .tp_getset      = py_flat_format_getseters,
        .tp_new         = (newfunc)py_flat_format_new

    };

    return &py_flat_format_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.format.FlatFormat'.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_flat_format(PyObject *module)
{
    PyTypeObject *py_flat_format_type;      /* Type Python 'FlatFormat'    */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_flat_format_type = get_python_flat_format_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_FLAT_FORMAT, py_flat_format_type, \
                                      get_python_executable_format_type()))
        return false;

    return true;

}
